"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vitest_1 = require("vitest");
const MessageHandler_1 = require("../../lib/MessageHandler");
const Schema_1 = require("../../lib/Schema");
const helper_1 = require("../helper");
function assertHover(position, expected, fixturePath) {
    const textDocument = (0, helper_1.getTextDocument)(fixturePath);
    const params = {
        textDocument,
        position: position,
    };
    const hoverResult = (0, MessageHandler_1.handleHoverRequest)(Schema_1.PrismaSchema.singleFile(textDocument), textDocument, params);
    (0, vitest_1.expect)(hoverResult).not.toBeUndefined();
    (0, vitest_1.expect)(hoverResult === null || hoverResult === void 0 ? void 0 : hoverResult.contents).toStrictEqual(expected.contents);
    (0, vitest_1.expect)(hoverResult === null || hoverResult === void 0 ? void 0 : hoverResult.range).toStrictEqual(expected.range);
}
(0, vitest_1.describe)('Hover of /// documentation comments', () => {
    const fixturePath = './hover/postgresql.prisma';
    (0, vitest_1.test)('Model', () => {
        assertHover({
            character: 15,
            line: 24,
        }, {
            contents: "Post including an author, it's content\n\nand whether it was published",
        }, fixturePath);
    });
    (0, vitest_1.test)('Enum', () => {
        assertHover({
            character: 15,
            line: 25,
        }, {
            contents: 'This is an enum specifying the UserName.',
        }, fixturePath);
    });
});
//# sourceMappingURL=single-file.test.js.map